/* 
** compat.c: 
** copyright: (c) 2012 by Hemmingsen Kim
** email: kmgh10@hotmail.com
** This program is free software; you can redistribute it and/or 
** modify  it under the terms of the GNU General Public License as
** published by the Free Software Foundation; either version 2 of
** the License, or (at your option) any later version.
*/
/*compat.c holds redefinition of gtk2 syntax to gtk1*/
#include "compat.h"
#include "pixmaps.h"

/// From ROX: Scale src down to fit in max_w, max_h and return the new pixbuf.
/// If src is small enough, then ref it and return that.
GdkPixbuf *scale_pixbuf(GdkPixbuf *src, int max_w, int max_h) {
	int	w, h;

	w = gdk_pixbuf_get_width(src);
	h = gdk_pixbuf_get_height(src);

	if (w <= max_w && h <= max_h) {
		gdk_pixbuf_ref(src);
		return src;
	} else {
		float scale_x = ((float) w) / max_w;
		float scale_y = ((float) h) / max_h;
		float scale = MAX(scale_x, scale_y);
		int dest_w = w / scale;
		int dest_h = h / scale;
		
		return gdk_pixbuf_scale_simple(src,	MAX(dest_w, 1),	MAX(dest_h, 1),
			GDK_INTERP_BILINEAR);
	}
}

/// From ROX: Scale src up to fit in max_w, max_h and return the new pixbuf.
/// If src is that size or bigger, then ref it and return that.
GdkPixbuf *scale_pixbuf_up(GdkPixbuf *src, int max_w, int max_h) {
	int	w, h;

	w = gdk_pixbuf_get_width(src);
	h = gdk_pixbuf_get_height(src);

	if (w == 0 || h == 0 || (w >= max_w && h >= max_h)) {
		gdk_pixbuf_ref(src);
		return src;
	} else {
		float scale_x = max_w / ((float) w);
		float scale_y = max_h / ((float) h);
		float scale = MIN(scale_x, scale_y);
		
		return gdk_pixbuf_scale_simple(src,	w * scale, h * scale,
			GDK_INTERP_BILINEAR);
	}
}

/// Create a new hbox with an image and a label packed into it
/// and return the box.
GtkWidget *xpm_label_box( GtkWidget *parent, char *xpm_filename, gchar *label_text, int height, int width ) {
	GtkWidget *box1;
	GtkWidget *label;
	GtkWidget *pixmapwid;
	GdkPixmap *pixmap;
	GdkBitmap *mask;
	GdkColormap *colormap;
	GdkPixbuf *pixbuf, *normal_pixbuf;
	FILE *file = NULL;
	/// for buildin images
	const char **xpm = NULL;
		
	box1 = gtk_hbox_new (FALSE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (box1), 2);
	colormap = gtk_widget_get_colormap (parent);
	
	file = fopen(xpm_filename, "rb");
	
	if (!file) {
		if (!label_text) {
			xpm = error_xpm;
		}
		/// find default build in image if external fail
		else if ( !strcmp(label_text, "Yes") ) 
			xpm = yes_xpm;
		else if (!strcmp(label_text, "OK") )
			xpm = ok_xpm;
		else if (!strcmp(label_text, "Cancel") || !strcmp(label_text, "No"))
			xpm = no_xpm;
		else if (!strcmp(label_text, "Help")) 
			xpm = help_xpm;		
		else
			xpm = error_xpm;
			
		pixbuf = gdk_pixbuf_new_from_xpm_data (xpm);
		normal_pixbuf = scale_pixbuf(pixbuf, SMALL_HEIGHT, SMALL_WIDTH);
		gdk_pixbuf_render_pixmap_and_mask(normal_pixbuf, &pixmap, &mask, 128);
		pixmapwid = gtk_pixmap_new (pixmap, mask);
		gdk_pixbuf_unref(normal_pixbuf);
		gdk_pixbuf_unref(pixbuf);
			
	} else {
		
		fclose(file);	
#ifdef USE_GTK2	
		pixbuf = gdk_pixbuf_new_from_file(xpm_filename, NULL);
#else
		pixbuf = gdk_pixbuf_new_from_file(xpm_filename);
#endif	

		if (( ! height == 0) && ( ! width == 0)) {
			//we scale down and up...
			normal_pixbuf = scale_pixbuf_up(pixbuf, height, width);
			normal_pixbuf = scale_pixbuf(pixbuf, height, width);
		} else {
			normal_pixbuf = scale_pixbuf(pixbuf, SMALL_HEIGHT, SMALL_WIDTH);
			
		}
		gdk_pixbuf_render_pixmap_and_mask(normal_pixbuf, &pixmap, &mask, 128);
		pixmapwid = gtk_pixmap_new (pixmap, mask);
		gdk_pixbuf_unref(normal_pixbuf);
		gdk_pixbuf_unref(pixbuf);
	}
		
	gtk_box_pack_start(GTK_BOX (box1), pixmapwid, FALSE, FALSE, 3);
	gtk_widget_show(pixmapwid);
	if ( ! (pixmap == NULL)) {	/// precaution static builds might segfault
		gdk_pixmap_unref(pixmap);
	}
	if ( ! (mask == NULL)) {	/// precaution static builds might segfault
		gdk_pixmap_unref(mask); 
	}
	/// skip label if not needed
	if (label_text) {
		label = gtk_label_new (label_text);
		gtk_box_pack_start(GTK_BOX (box1), label, FALSE, FALSE, 3);
		gtk_widget_show(label);
	}
	
	return(box1);
}

#ifndef USE_GTK2
#define _GNU_SOURCE
#include <stdio.h>
#include "attributes.h"
#include <sys/stat.h>	//needed if we use "stat"
#include <ctype.h>	//needed for 'toupper'

/// gtk1 wrapper
void gtk_window_present(GtkWindow *window) {
	return;
}

/// gtk1 wrapper
GtkWidget *gtk_text_view_new(void) {
	GtkWidget	*Widget = NULL;
	Widget=gtk_text_new(NULL,NULL);
	gtk_text_set_editable ( GTK_TEXT (Widget), TRUE);
	gtk_text_set_word_wrap( GTK_TEXT (Widget), TRUE);
	gtk_text_set_point( GTK_TEXT (Widget), 0);
		    	
	return(Widget);
}

/// gtk1 wrapper
GtkWidget *gtk_button_new_from_stock (char *stock_id) {
	
	GtkWidget	*Widget = NULL;
	GtkWidget	*box1 = NULL;
	char *label;
	char *filename;
	
	if ( !strcmp(stock_id, GTK_STOCK_YES) ) 
			label = "Yes";
		else if ( !strcmp(stock_id, GTK_STOCK_OK) )
			label = "OK";
		else if ( !strcmp(stock_id, GTK_STOCK_CANCEL) )
			label = "Cancel";
		else if ( !strcmp(stock_id, GTK_STOCK_NO) )
			label = "No";
		else if ( !strcmp(stock_id, GTK_STOCK_HELP) ) 
			label = "Help";		
		else
			label = "Unlabeled";
		
	filename = find_pixmap(stock_id);
	Widget = gtk_button_new();
	
    box1 = xpm_label_box(Widget, filename, label, 0, 0);
    gtk_container_add (GTK_CONTAINER (Widget), box1);

	return(Widget);
}


/// my helpers...
char *replace_str(char *str, char *orig, char *rep) {
  static char buffer[4096];
  char *p;

  if(!(p = strstr(str, orig)))  // Is 'orig' even in 'str'?
    return str;

  strncpy(buffer, str, p-str); // Copy characters from 'str' start to 'orig' st$
  buffer[p-str] = '\0';

  sprintf(buffer+(p-str), "%s%s", rep, p+strlen(orig));

  return buffer;
}



/// gtk1 wrapper
GtkWidget *gtk_image_menu_item_new_from_stock (char *stock_id, int *dummy) {
	
	GtkWidget	*menu_item = NULL;
	GtkWidget	*hbox;
	GtkWidget	*label;	
	GtkWidget	*img;
	char		*label_old = stock_id;
	
	if (strncmp("gtk-", stock_id, 4) == 0){
		label_old = replace_str(label_old, "gtk-", "");
		label_old[0] = toupper(label_old[0]); /* capitalize */
	}	
			
	hbox = gtk_hbox_new(FALSE, 0);	
	label = gtk_label_new(label_old);
	img = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_BUTTON, GTK_ICON_SIZE_BUTTON);
	gtk_box_pack_start(GTK_BOX(hbox), img, FALSE, TRUE, 0); 
  	gtk_widget_show(img);
  	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);
	gtk_widget_show(label);
	
	menu_item = gtk_menu_item_new();
	gtk_container_add(GTK_CONTAINER(menu_item), hbox);
	gtk_widget_show(hbox);
			
	return menu_item;
}

///gtk1 wrapper
GtkWidget *gtk_image_new_from_file(char *str, int height, int width) {
	GtkWidget *Icon = NULL;
	GtkWidget *Box = NULL;
		
	Icon = gtk_hbox_new(FALSE, 5);
	Box = xpm_label_box(Icon, (find_pixmap(str)), NULL, height, width);
	gtk_container_add (GTK_CONTAINER (Icon), Box);
	
	return (Icon);
}

///gtk1 wrapper
GtkWidget *gtk_image_new_from_stock (char *stock_id, int height, int width) {
	
	GtkWidget *Icon = NULL;
	GtkWidget *Box = NULL;
	char *filename = NULL;
	Icon = gtk_hbox_new(FALSE, 5);
	
	///the stock=" syntax
	if (strncmp("gtk-", stock_id, 4) == 0){
		filename=replace_str(stock_id, "gtk-", "stock_");
		filename=replace_str(filename, "-", "_");
		asprintf(&filename, "%s%s", filename, "_24.png");
	} else {
	///the icon=" syntax - hopefully - as we dont check!
		asprintf(&stock_id, "%s%s", stock_id, ".png");
		filename = stock_id;
	}
	
	Box = xpm_label_box(Icon, (find_pixmap(filename)), NULL, height, width);
	gtk_container_add (GTK_CONTAINER (Icon), Box);
	
	return (Icon);
}

void action_chooserselection_made( GtkWidget *w, GtkFileSelection *fs ) {
	char *selected_path;
	selected_path = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
	CHOOSERCONT = selected_path;
	return;
}

void action_dirselected (GtkWidget *widget, gint row, gint column, GdkEventButton *bevent, gpointer user_data) {
	char *selected_dir,*temp = NULL;
	GtkFileSelection *fs = NULL;
	fs = GTK_FILE_SELECTION (user_data);
	gtk_clist_get_text (GTK_CLIST (fs->dir_list), row, 0, &temp);
  	selected_dir = g_strdup (temp);
  	CHOOSERCONT = selected_dir;
	return;
}

void action_fileselected (GtkWidget *widget, gint row, gint column, GdkEventButton *bevent, gpointer user_data) {
	char *selected_file,*temp = NULL;
	GtkFileSelection *fs = NULL;
	fs = GTK_FILE_SELECTION (user_data);
	gtk_clist_get_text (GTK_CLIST (fs->file_list), row, 0, &temp);
  	selected_file = g_strdup (temp);
  	CHOOSERCONT = selected_file;
	return;
}

#endif


